@@ -1,16 +1,22 @@
-;;; isearch mode for skk with emacs 19.
-
-;; $Id: skk-isearch.el,v 1.3 1997/09/16 08:56:20 mrt Exp $
-
-;; Copyright (C) 1994, 1995, 1996, 1997
+;; -*- byte-compile-dynamic-docstring: t;-*-
+;;; skk-isearch.el --- isearch mode for skk with Emacs 19 or later and XEmacs.
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999
 ;; Enami Tsugutomo <enami@ba2.so-net.or.jp>
 
-;; This program is free software; you can redistribute it and/or modify
+;; Author: Enami Tsugutomo <enami@ba2.so-net.or.jp>
+;; Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
+;; Version: $Id: skk-isearch.el,v 1.2 2000/07/10 04:34:00 yoshiki Exp $
+;; Keywords: japanese
+;; Last Modified: $Date: 2000/07/10 04:34:00 $
+
+;; This file is part of SKK.
+
+;; SKK is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either versions 2, or (at your option)
 ;; any later version.
 
-;; This program is distributed in the hope that it will be useful
+;; SKK is distributed in the hope that it will be useful
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -20,53 +26,139 @@
 ;; Software Foundation Inc., 59 Temple Place - Suite 330, Boston,
 ;; MA 02111-1307, USA.
 
+;;; Commentary:
+;; functions for hooks.
+;;
+;; 1. always invoke skk isearch.
+;; (add-hook 'isearch-mode-hook 
+;;           (function (lambda () (require 'skk) (skk-isearch-mode-setup))))
+;; (add-hook 'isearch-mode-end-hook 'skk-isearch-mode-cleanup)
+;;
+;; 2. invoke only if skk-mode is on.
+;; (add-hook 'isearch-mode-hook
+;;           (function (lambda ()
+;;			 (and (boundp 'skk-mode) skk-mode
+;;			      (skk-isearch-mode-setup)))))
+;;
+;; (add-hook 'isearch-mode-end-hook
+;;           (function
+;;            (lambda ()
+;;              (and (boundp 'skk-mode) skk-mode (skk-isearch-mode-cleanup)))))
+;; 
+;; 3. invoke if current buffer has japanese characters.
+;; ...
+;;
 ;; skk-isearch-initial-mode examine the variable of skk before calling
 ;; skk-mode.
+
+;;; Code:
+(require 'skk)
 (require 'skk-foreword)
-(require 'skk-vars)
 
-;;
-;; variables
-;;
-(defvar skk-isearch-incomplete-message "" "incomplete isearch message")
-(defvar skk-isearch-working-buffer " *skk-isearch*"
-  "work buffer for skk isearch.")
+;; user variables
+;;;###autoload
+(defgroup skk-isearch nil "SKK incremental search related customization."
+  :prefix "skk-isearch-"
+  :group 'skk )
 
-;; (setq skk-isearch-mode-map nil)
-;; (lookup-key skk-isearch-mode-map " ")
-(defvar skk-isearch-mode-map nil
-  "Keymap for skk isearch mode.  This map should be derived from
-isearch-mode-map.")
+(defcustom skk-isearch-mode-string-alist
+  '((hiragana . "[$B$+(B] ") (katakana . "[$B%+(B] ") (jisx0208-latin . "[$B1Q(B] ")
+    (latin . "[aa] ") (nil . "[--] "))
+  ;;  "*Alist of \(MODE-SYMBOL . PROMPT-STRING\).
+  ;;MODE-SYMBOL is a symbol indicates canonical mode of skk for skk-isearch.
+  ;;Valid MODE-SYMBOL is one of `hiragana', `katakana', `jisx0208-latin',
+  ;;`latin' or nil.
+  ;;PROMPT-STRING is a string used in prompt to indicates current mode of
+  ;;skk for skk-isearch. "
+  "*isearch $B;~$KF~NO%b!<%I$K=>$$=P$9%W%m%s%W%H;XDj$N$?$a$N%(!<%j%9%H!#(B
+$B3FMWAG$O!"(B
 
-;;;###skk-autoload
-(defvar skk-isearch-whitespace-regexp "\\(\\s \\|[ \t\n\r\f]\\)*")
+  \(MODE-SYMBOL . PROMPT-STRING\)
 
-(defvar skk-isearch-overriding-local-map
-  (cond (skk-xemacs
-	 'overriding-terminal-local-map)
-	((and (boundp 'emacs-major-version)
-	      (or (> emacs-major-version 19)
-		  (and (= emacs-major-version 19) 
-		       (>= emacs-minor-version 29))))
-	 ;; GNU Emacs version 19.29, 19.30 and 19.31 uses this in isearch.el.
-	 'overriding-terminal-local-map)
-	;; GNU Emacs version 19.22 .. 19.28 uses this in isearch.el.
-	((string-match "^19\\.2[2-8]" emacs-version) 'overriding-local-map))
-  "Variable holding overrinding local map used in isearch-mode.")
+$B$H$$$&(B cons cell$B!#(B
+MODE-SYMBOL $B$OF~NO%b!<%I$rI=$o$9%7%s%\%k$G!"(B
+$B2<5-$N$$$:$l$+$r;XDj$9$k!#(B
 
-(defvar skk-isearch-breakup-string-function
-  (cond ((fboundp 'string-to-char-list)
-	 ;; Mule 2.3 or its ancestor defines `string-to-char-list'.
-	 'string-to-char-list)
-	((fboundp 'string-to-list)
-	 ;; delta version of Mule merged Emacs currently defines
-	 ;; `string-to-list'.
-	 'string-to-list)
-	(t (error "No appropriate function as: %s"
-		  'skk-isearch-breakup-string-function)))
-  "Function to breakup STRING into list of characters.")
+   $B$+$J%b!<%I!'(B `hiragana'
+   $B%+%J%b!<%I!'(B `katakana'
+   $BA41Q%b!<%I!'(B `jisx0208-latin'
+   $B%"%9%-!<%b!<%I!'(B `latin'
 
-(defvar skk-isearch-breakable-character-p-function
+nil $B$O!"(BSKK $B%b!<%I%*%U$rI=$o$9!#(B
+PROMPT-STRING $B$O!"3:Ev$N(B SKK $B%b!<%I$KBP$7=P$9%W%m%s%W%H$NJ8;zNs!#(B"
+  :type '(repeat (cons (choice :tag "Mode symbol"
+			       (const hiragana)
+			       (const katakana)
+			       (const jisx0208-latin)
+			       (const latin)
+			       (const nil))
+		       (string :tag "Prompt string")))
+  :group 'skk-isearch )
+
+(defcustom skk-isearch-start-mode nil
+  ;;  "*Specifies the search mode when isearch is called.
+  ;;This variable is valid only when `skk-isearch-use-previous-mode' is nil.
+  ;;If nil, it means that if skk-mode has been called in this buffer, same as
+  ;;the mode of the buffer, otherwise perform ascii search.
+  ;;If `latin' or `ascii' perfrom ascii search.
+  ;;If `hiragana', `hirakana' or `kana' -> hira kana search.
+  ;;If `jisx0208-latin' or `eiji', perform zenkaku eiji (i.e. JIS X0208 alphabet) search."
+  "*$B%+%l%s%H%P%C%U%!$G(B isearch $B$r9T$J$&:]$NF~NO%b!<%I!#(B
+`skk-isearch-use-previous-mode' $B$,(B nil $B$N>l9g$N$_M-8z!#(B
+isearch $B$r9T$J$&>l9g!">o$K$3$NJQ?t$G;XDj$7$?F~NO%b!<%I$,;HMQ$5$l$k(B ($B%f!<%6!<$,(B
+$BL@<(E*$KJQ99$r9T$J$&$3$H$O2D(B)$B!#(B
+$B2<5-$N$$$:$l$+$N%7%s%\%k$G;XDj$9$k!#(B
+
+   nil:  $B%+%l%s%H%P%C%U%!$G(B SKK $B%b!<%I$,5/F0$5$l$F$$$l$P$=$N%b!<%I!"(B
+         $B5/F0$5$l$F$$$J$1$l$P(B $B%"%9%-!<%b!<%I!#(B
+   `hiragana' (`hiragana' or `kana'): $B$+$J%b!<%I(B
+   `jisx0208-latin' (`eiji') : $BA41Q%b!<%I(B
+   `latin' (`ascii'): $B%"%9%-!<%b!<%I(B"
+  :type '(choice (const :tag "Succeed an input mode of current buffer" nil)
+		 (const :tag "Ascii search" latin)
+		 (const :tag "Hiragana search" hiragana)
+		 (const :tag "JISX0208 alphabet search" jisx0208-latin))
+  :group 'skk-isearch )
+
+(defcustom skk-isearch-use-previous-mode nil
+  ;; "*Non-nil means use same search mode as the search mode of the last search in the buffer."
+  "*Non-nil $B$G$"$l$P!"%+%l%s%H%P%C%U%!$G:G8e$K9T$J$C$?(B isearch $B$N(B SKK $B%b!<%I$HF1$8%b!<%I$r;HMQ$9$k!#(B"
+  :type 'boolean
+  :group 'skk-isearch )
+
+(defcustom skk-isearch-initial-mode-when-skk-mode-disabled 'latin
+  ;;  "*Symbol indicates the mode to use as initial mode for skk-isearch when
+  ;;skk is turned off in the current buffer."
+  "*SKK $B%b!<%I$,%*%U$N%+%l%s%H%P%C%U%!$G!":G=i$K(B isearch $B$r9T$J$&:]$NF~NO%b!<%I!#(B"
+  :type '(choice (const :tag "Ascii search" latin)
+		 (const :tag "Hiragana search" hiragana)
+		 (const :tag "JISX0208 alphabet search" jisx0208-latin))
+  :group 'skk-isearch )
+
+(defcustom skk-isearch-whitespace-regexp "\\(\\s \\|[ \t\n\r\f]\\)*"
+  ;;  "*Regular expression to match a sequence of whitespace chars.
+  ;;This applies to regular expression incremental search."
+  "$B6uGrJ8;z$NO"B3$H$7$F%^%C%A$5$;$k$Y$-@55,I=8=!#(B
+regexp isearch $B$N:]!"$3$N@55,I=8=$K%^%C%A$9$kJ8;z$,8!:wJ8;zNs$N4V$K4^$^$l$F$$$F(B
+$B$b%^%C%A$9$k!#(B"
+  :type 'regexp
+  :group 'skk-isearch )
+
+;; internal constants and variables.
+(defconst skk-isearch-mode-canonical-alist
+  '((hiragana . 0) (katakana . 1) (jisx0208-latin . 2) (latin . 3))
+  "Alist of \(SYMBOL . NUMBER\).
+The SYMBOL is canonical skk mode, and NUMBER is its numerical representation.")
+
+(defconst skk-isearch-mode-alias-alist
+  '((hirakana . hiragana) (kana . hiragana) (eiji . jisx0208-latin)
+    (ascii . latin))
+  "Alist of \(ALIAS . CANONICAL\).
+The both ALIAS and CANONICAL should be symbol.
+ALIAS can be used as an alias of CANONICAL.
+CANONICAL should be found in `skk-isearch-mode-canonical-alist'. ")
+
+(defconst skk-isearch-breakable-character-p-function
   (cond ((fboundp 'char-category-set)
 	 (function (lambda (char)
 		     ;; see emacs/lisp/fill.el how the category `|' is
@@ -76,45 +168,62 @@ isearch-mode-map.")
 	 (function (lambda (char)
 		     ;; (let ((lc (char-leading-char char)))
 		     ;;   (or (= lc lc-jp) (= lc lc-cn)))
-		     (string-match (char-to-string char)
-				   word-across-newline))))
+		     (string-match word-across-newline
+				   (char-to-string char)))))
 	(t (error "No appropriate function as: %s"
 		  'skk-isearch-breakable-character-p-function)))
-  "Function to test if we can insert a newline around the CHAR when filling.")
+  "Function to test if we can insert a newline around CHAR when filling.")
 
-(defvar skk-isearch-use-previous-mode nil
-  "*If non-nil, search mode will be same as the last search mode for the
-previous search in the buffer.")
-
-;; suggested by Yoshiyuki Kondo, 1991.6.19.
-;; modified by Mikio Nakajima, 1995.5.30
-(defvar skk-isearch-start-mode nil
-  "*Specifies the search mode when isearch is called.
-This variable is valid only when `skk-isearch-use-previous-mode' is nil.
-If nil, it means that if skk-mode has been called in this buffer, same as
-the mode of the buffer, otherwise perform ascii search.  If `ascii', perfrom
-ascii search.  If `kana' or `hirakana' -> hira kana search.  If `eiji',
-perform zenkaku eiji (i.e. JIS X0208 alphabet) search.")
+(defconst skk-isearch-working-buffer " *skk-isearch*"
+  "Work buffer for skk isearch." )
 
 (defvar skk-isearch-mode nil
-  "Current search mode.  0 -> hira kana search.  1 -> kata kana search.
-2 -> zenkaku eiji (i.e. JIS X0208 alphabet) search.  3 -> ascii search.")
+  "Current search mode.
+0 means hira kana search.
+1 means kana search.
+2 means zenkaku eiji (i.e. JIS X0208 alphabet) search.
+3 means ascii search." )
 
-
-(defsubst skk-isearch-set-working-buffer ()
-  "Set current buffer to the working buffer for skk isearch."
-  (set-buffer (get-buffer-create skk-isearch-working-buffer)))
+(defvar skk-isearch-incomplete-message ""
+  "Incomplete isearch message" )
+
+(defvar skk-isearch-mode-map nil
+  "Keymap for skk isearch mode.
+This map should be derived from isearch-mode-map." )
+
+(defvar skk-isearch-overriding-local-map
+  (cond ((eq skk-emacs-type 'xemacs)
+	 (cond
+	  ((or (> emacs-major-version 21)
+	       (and (= emacs-major-version 21)
+		    (or (> emacs-minor-version 2)
+			(and (= emacs-minor-version 2)
+			     (boundp 'emacs-beta-version) emacs-beta-version
+			     (>= emacs-beta-version 2)))))
+	   'overriding-local-map )
+	  (t 'overriding-terminal-local-map)))
+	;; for Mule/GNU Emacs.
+	((or (> emacs-major-version 19)
+	     (and (= emacs-major-version 19) (> emacs-minor-version 28)))
+	 ;; GNU Emacs version 19.29, 19.30 and 19.31 uses this in isearch.el.
+	 'overriding-terminal-local-map )
+	;; GNU Emacs version 19.22 .. 19.28 uses this in isearch.el.
+	(t 'overriding-local-map))
+  "Variable holding overrinding local map used in isearch-mode.")
+
+(defvar skk-isearch-last-mode-string "")
+(defvar skk-isearch-last-mode-regexp "")
 
 ;;
 ;; interface to skk.el
 ;;
 (defsubst skk-isearch-turn-off-skk-mode ()
   "Turn off skk mode."
-  (skk-mode 0) )
+  (skk-mode 0))
 
 (defsubst skk-isearch-turn-on-skk-mode ()
   "Turn on skk mode."
-  (skk-mode 1) )
+  (skk-mode 1))
 
 (defsubst skk-isearch-conversion-active-p ()
   "Non-nil if skk conversion is active."
@@ -126,53 +235,47 @@ perform zenkaku eiji (i.e. JIS X0208 alphabet) search.")
 
 (defsubst skk-isearch-skk-kakutei ()
   "Perform kakutei."
-  (skk-kakutei) )
+  (skk-kakutei))
 
-(defsubst skk-isearch-skk-hirakana-mode-p ()
-  "Non-nil if skk is hirakana input mode."
-  (and (not skk-katakana) skk-j-mode) )
- 
-(defsubst skk-isearch-skk-turn-on-hirakana-mode ()
-  "Set current skk mode to hirakana input mode."
-  (skk-j-mode-on) )
+(defsubst skk-isearch-skk-hiragana-mode-p ()
+  "Non-nil if skk is hiragana input mode."
+  (and (not skk-katakana) skk-j-mode))
+
+(defsubst skk-isearch-skk-turn-on-hiragana-mode ()
+  "Set current skk mode to hiragana input mode."
+  (skk-j-mode-on))
 
 (defsubst skk-isearch-skk-katakana-mode-p ()
   "Non-nil if skk is katakana input mode."
-  (and skk-j-mode skk-katakana) )
+  (and skk-j-mode skk-katakana))
 
 (defsubst skk-isearch-skk-turn-on-katakana-mode ()
   "Set current skk mode to katakana input mode."
-  (skk-j-mode-on 'katakana) )
+  (skk-j-mode-on 'katakana))
 
-(defsubst skk-isearch-skk-jix0208-latin-mode-p ()
-  "Non-nil if skk is zenkaku (jisx0208 latin) input mode."
-  skk-zenkaku-mode )
+(defsubst skk-isearch-skk-jisx0208-latin-mode-p ()
+  "Non-nil if skk is jisx0208 latin (zenkaku) input mode."
+  skk-jisx0208-latin-mode )
 
 (defsubst skk-isearch-skk-turn-on-jix0208-latin-mode ()
-  "Set current skk mode to zenkaku (jisx0208 latin) input mode."
-  (skk-zenkaku-mode-on) )
+  "Set current skk mode to jisx0208 latin (zenkaku) input mode."
+  (skk-jisx0208-latin-mode-on))
 
 (defsubst skk-isearch-skk-turn-on-latin-mode ()
   "Set current skk mode to normal latin input mode."
-  (skk-ascii-mode-on) )
+  (skk-latin-mode-on))
 
-;; Override the function in skk-isearch.el.  I hope the
-;; skk-kana-input would be rewritten without using while
-;; read-char loop...
 (defun skk-isearch-message ()
   "Show isearch message."
-  ;; `prefix' is dynmaic variable in skk-kana-input.
   (skk-isearch-incomplete-message
-   (if (boundp 'prefix)
-       prefix
-     (char-to-string last-command-char))))
+   (if (string= skk-prefix "") (char-to-string last-command-char) skk-prefix)))
 
 (defun skk-isearch-current-mode ()
   "Return the symbolic current mode of skk for skk-isearch."
   (cond ((not skk-mode) nil)
 	((skk-isearch-skk-katakana-mode-p) 'katakana)
-	((skk-isearch-skk-hirakana-mode-p) 'hiragana)
-	((skk-isearch-skk-jix0208-latin-mode-p) 'jisx0208-latin)
+	((skk-isearch-skk-hiragana-mode-p) 'hiragana)
+	((skk-isearch-skk-jisx0208-latin-mode-p) 'jisx0208-latin)
 	(t 'latin)))
 
 (defun skk-isearch-set-initial-mode (mode)
@@ -181,7 +284,7 @@ The MODE should be canonical."
   ;; following code is highly depends on internal of skk.
   ;; (skk-isearch-turn-on-skk-mode)
   ;; (skk-isearch-skk-kakutei)
-  (cond ((eq mode 'hiragana) (skk-isearch-skk-turn-on-hirakana-mode))
+  (cond ((eq mode 'hiragana) (skk-isearch-skk-turn-on-hiragana-mode))
 	((eq mode 'katakana) (skk-isearch-skk-turn-on-katakana-mode))
 	((eq mode 'jisx0208-latin)
 	 (skk-isearch-skk-turn-on-jix0208-latin-mode))
@@ -190,45 +293,6 @@ The MODE should be canonical."
 	;; shouldn't happen.
 	(t (error "unknown skk-isearch-mode %s" mode))))
 
-;;
-;; functions for hooks.
-;;
-;; 1. always invoke skk isearch.
-;; (add-hook 'isearch-mode-hook 'skk-isearch-mode-setup)
-;; (add-hook 'isearch-mode-end-hook 'skk-isearch-mode-cleanup)
-;; 2. invoke only if skk-mode is on.
-;; (add-hook 'isearch-mode-hook
-;;           (function (lambda ()
-;;			 (and (boundp 'skk-mode) skk-mode
-;;			      (skk-isearch-mode-setup)))))
-;; (add-hook 'isearch-mode-end-hook
-;;	     (function (lambda ()
-;;			 (and (boundp 'skk-mode) skk-mode
-;;			      (skk-isearch-mode-cleanup)))))
-;; 3. invoke if current buffer has japanese characters.
-;; ...
-
-(defvar skk-isearch-mode-canonical-alist
-  '((hiragana . 0) (katakana . 1) (jisx0208-latin . 2) (latin . 3))
-  "List of dot pair, (SYMBOL . NUMBER).
-The SYMBOL is canonical skk mode, and NUMBER is its numerical representation.")
-
-(defvar skk-isearch-mode-alias-alist
-  '((hirakana . hiragana) (kana . hiragana)
-    (eiji . jisx0208-latin)
-    (ascii . latin))
-  "List of dot pair, (ALIAS . CANONICAL).  The both ALIAS and CANONICAL should
-be symbol.  ALIAS can be used as an alias of CANONICAL.  CANONICAL should be
-found in `skk-isearch-mode-canonical-alist'.")
-
-;; (makunbound 'skk-isearch-mode-string-alist)
-(defvar skk-isearch-mode-string-alist
-  '((hiragana . "[$B$+(B] ") (katakana . "[$B%+(B] ")
-    (jisx0208-latin . "[$B1Q(B] ") (latin . "[aa] ") (nil . "[--] "))
-  "Alist of (MODE-SYMBOL . PROMPT-STRING).  MODE-SYMBOL is a symbol
-indicates canonical mode of skk for skk-isearch.  PROMPT-STRING is a string
-used in prompt to indicates current mode of skk for skk-isearch.")
-
 (defun skk-isearch-symbolic-mode (mode)
   "Return symbolic skk isearch mode for given numerical MODE."
   (car (rassq mode skk-isearch-mode-canonical-alist)))
@@ -250,14 +314,11 @@ internal condition."
 (defun skk-isearch-canonical-start-mode (mode)
   "Canonicalize the symbolic skk isearch MODE."
   ;; alias, canonical, or error.
-  (cond ((cdr (assq mode skk-isearch-mode-alias-alist)))
+  (cond ((cdr (rassq mode skk-isearch-mode-alias-alist)))
+	((cdr (assq mode skk-isearch-mode-alias-alist)))
 	((skk-isearch-numerical-mode mode) mode)
 	(t (error "Unknown skk-isearch-start-mode: %s" mode))))
 
-(defvar skk-isearch-initial-mode-when-skk-mode-disabled 'latin
-  "*Symbol indicates the mode to use as initial mode for skk-isearch when
-skk is turned off in the current buffer.")
-
 (defun skk-isearch-initial-mode ()
   "Return the symbolic mode name of skk-isearch used to initialize working
 buffer."
@@ -292,6 +353,7 @@ kakutei'ed and erase the buffer contents."
   ;; determined in the original buffer and set in working buffer.
   (let ((initial (skk-isearch-initial-mode)))
     (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
+      (skk-erase-prefix 'clean)
       (skk-isearch-initialize-working-buffer)
       (skk-isearch-set-initial-mode initial)))
   ;; setup variables and keymap
@@ -306,24 +368,33 @@ kakutei'ed and erase the buffer contents."
 (defun skk-isearch-mode-cleanup ()
   "Hook function called when skk isearch is done."
   ;; remember the current skk mode for next use.
-  (and skk-isearch-use-previous-mode
-       (setq skk-isearch-mode
-	     (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
-	       (skk-isearch-current-numerical-mode))))
-  ;; reset the overrinding-local-map.
-  (set skk-isearch-overriding-local-map nil)
-  (setq skk-isearch-message nil
-	skk-isearch-last-mode-string ""
-	skk-isearch-last-mode-regexp ""))
-
-;;; for backward compatibility
-(defalias 'skk-isearch-forward 'isearch-forward)
-(defalias 'skk-isearch-forward-regexp 'isearch-forward-regexp)
-(defalias 'skk-isearch-backward 'isearch-backward)
-(defalias 'skk-isearch-backward-regexp 'isearch-backward-regexp)
+  (let ((mode (skk-current-insert-mode)))
+    (and skk-isearch-use-previous-mode
+	 (setq skk-isearch-mode
+	       (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
+		 (skk-isearch-current-numerical-mode))))
+    ;; reset the overrinding-local-map.
+    (set skk-isearch-overriding-local-map nil)
+    (setq skk-isearch-message nil
+	  skk-isearch-last-mode-string ""
+	  skk-isearch-last-mode-regexp "" )
+    ;; $B%5!<%ACf$KF~NO%b!<%I$rJQ99$7$?$i!"%b!<%I%i%$%s$NI=<($b$=$l$K=>$$(B
+    ;; $BJQ99$5$l$k$N$G!"%+%l%s%H%P%C%U%!$NF~NO%b!<%I$H%b!<%I%i%$%s$NI=<((B
+    ;; $B$H$,(B sync $B$7$J$/$J$k!#=>$$!"%5!<%A$,=*N;$7$?:]!"%b!<%I%i%$%s$r%+(B
+    ;; $B%l%s%H%P%C%U%!$NF~NO%b!<%I$H(B sync $B$5$;$k!#(B
+    (cond ((eq mode 'hiragana) (skk-j-mode-on))
+	  ((eq mode 'katakana) (skk-j-mode-on t))
+	  ((eq mode 'abbrev) (skk-abbrev-mode-on))
+	  ((eq mode 'latin) (skk-latin-mode-on))
+	  ((eq mode 'jisx0208-latin) (skk-jisx0208-latin-mode-on))))
+  (remove-hook 'pre-command-hook 'skk-pre-command 'local)
+  (skk-remove-minibuffer-setup-hook
+   'skk-j-mode-on 'skk-setup-minibuffer
+   (function (lambda ()
+	       (add-hook 'pre-command-hook 'skk-pre-command nil 'local)))))
 
 (defun skk-isearch-incomplete-message (&optional prefix)
-  "Show message when when kana kanji convertion is progress.
+  "Show message when kana kanji convertion is in progress.
 Optional argument PREFIX is apppended if given."
   (let ((isearch-message (concat isearch-message
 				 skk-isearch-incomplete-message prefix)))
@@ -354,12 +425,12 @@ Optional argument PREFIX is apppended if given."
   map)
 
 (or skk-isearch-mode-map
-    (if skk-xemacs
+    (if (eq skk-emacs-type 'xemacs)
         (progn
           (setq skk-isearch-mode-map (skk-isearch-setup-keymap (make-keymap)))
-          (set-keymap-parents skk-isearch-mode-map isearch-mode-map) )
+          (set-keymap-parents skk-isearch-mode-map isearch-mode-map))
       (setq skk-isearch-mode-map
-            (skk-isearch-setup-keymap (cons 'keymap isearch-mode-map)) )))
+            (skk-isearch-setup-keymap (cons 'keymap isearch-mode-map)))))
 
 
 ;;
@@ -374,7 +445,7 @@ Optional argument PREFIX is apppended if given."
 	(progn
 	  ;; temporarily disable the overriding-local-map.  this
 	  ;; should be done in ther buffer isearch is performed, i.e.,
-	  ;; before calling skk-isearch-set-working-buffer.
+	  ;; before entering skk-isearch-working-buffer.
 	  (set skk-isearch-overriding-local-map nil)
 	  ;; don't change the current buffer during save/restore the
 	  ;; overriding-local-map, because it is buffer local in some
@@ -383,10 +454,10 @@ Optional argument PREFIX is apppended if given."
 	    ;; listify this-command-keys.  this works only if it is
 	    ;; string.
 	    (setq unread-command-events
-		  (append (if (= (length (this-command-keys)) 0)
-			      (list last-command-event)
-			    (this-command-keys) )
-			  nil ))
+                  (append (if (= (length (this-command-keys)) 0)
+                              (list last-command-event)
+                            (this-command-keys))
+                          nil ))
 	    (condition-case error
 		;; setup last-command-event and this-command because
 		;; some command refers them.
@@ -401,7 +472,7 @@ Optional argument PREFIX is apppended if given."
 
 (defun skk-isearch-search-string ()
   "Return the string to be searched.
-If the conversion is progress and no string is fixed, just return nil."
+If the conversion is in progress and no string is fixed, just return nil."
     (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
       (prog1
 	  (cond ((skk-isearch-conversion-active-p)
@@ -411,6 +482,9 @@ If the conversion is progress and no string is fixed, just return nil."
 		       (prog1
 			   (buffer-substring 1 start)
 			 (delete-region 1 start)))))
+		;; a conversion is in progress
+		((not (string= skk-prefix "")) nil)
+		;;(skk-current-rule-tree nil)
 		;; whole string in the buffer is fixed.
 		((not (zerop (buffer-size)))
 		 (prog1
@@ -435,7 +509,7 @@ If the conversion is progress and no string is fixed, just return nil."
 
 (defun skk-isearch-search-string-regexp (string)
   (if isearch-regexp
-      (let ((chars (funcall skk-isearch-breakup-string-function string))
+      (let ((chars (string-to-char-list string))
 	    (prev (skk-isearch-last-char isearch-string))
 	    (result ""))
 	(while chars
@@ -448,9 +522,6 @@ If the conversion is progress and no string is fixed, just return nil."
 	result)
     string))
 
-(defvar skk-isearch-last-mode-string "")
-(defvar skk-isearch-last-mode-regexp "")
-
 (defun skk-isearch-mode-message ()
   "Prepend the skk isearch mode string to `isearch-message'.  If the current
 mode is different from previous, remove it first."
@@ -460,12 +531,12 @@ mode is different from previous, remove it first."
       (if (string-match skk-isearch-last-mode-regexp isearch-message)
 	  (setq isearch-message (substring isearch-message (match-end 0))))
       (setq skk-isearch-last-mode-string mode-string
-	    skk-isearch-last-mode-regexp (concat "^" 
+	    skk-isearch-last-mode-regexp (concat "^"
 						 (regexp-quote mode-string)))
       (setq isearch-message (concat mode-string isearch-message)))))
 
 (defun skk-isearch-process-search-string (string)
-  (isearch-process-search-string (skk-isearch-search-string-regexp string) 
+  (isearch-process-search-string (skk-isearch-search-string-regexp string)
 				 string))
 
 
@@ -479,9 +550,11 @@ mode is different from previous, remove it first."
 	(if (skk-isearch-conversion-active-p)
 	    (prog1
 		t
-	      ;; now, we can't pass the universal argument within the
-	      ;; isearch-mode.  so hard code the value `1'.
-	      (delete-backward-char 1)
+	      (if (string= skk-prefix "")
+		  ;; now, we can't pass the universal argument within the
+		  ;; isearch-mode.  so hard code the value `1'.
+		  (delete-backward-char 1)
+		(skk-erase-prefix 'clean))
 	      (setq skk-isearch-incomplete-message (buffer-string))
 	      (skk-isearch-incomplete-message))))
       (isearch-delete-char)))
@@ -511,7 +584,7 @@ mode is different from previous, remove it first."
 	      ;; if the working buffer is latin or jisx0208-latin
 	      ;; mode, default behaviour of C-j is set current mode
 	      ;; to kana mode.
-	      (skk-mode 1)
+	      (skk-isearch-turn-on-skk-mode)
 	      (skk-isearch-mode-message))))
       (isearch-message)
     (skk-isearch-kakutei (function isearch-printing-char))))
@@ -546,11 +619,21 @@ mode is different from previous, remove it first."
 	    (progn
 	      (set skk-isearch-overriding-local-map isearch-mode-map)
 	      (let ((command (key-binding string)))
-		(if (commandp command)
-		    ;; process a special character, such as *, |, ...
-		    (command-execute command)
-		  ;; just search literally.
-		  (skk-isearch-process-search-string string) )))
+		(cond ((not (commandp command))
+		       ;; just search literally.
+		       (skk-isearch-process-search-string string))
+		      ;; internationalized isearch.el
+		      ((fboundp 'isearch-process-search-multibyte-characters)
+		       ;; internationalized isearch.el binds all
+		       ;; multibyte characters to `isearch-printing-char'.
+		       (cond (
+			      ;; STRING is not a multibyte character.
+			      (> skk-kanji-len (length string))
+			      ;; process a special character, such as *, |, ...
+			      (command-execute command))
+			     (t (skk-isearch-process-search-string string))))
+		      ;; non internationalized isearch.el
+		      (t (command-execute command)))))
 	  ;; restore the overriding local map.
 	  (set-buffer current-buffer)
 	  (set skk-isearch-overriding-local-map local-map))))))
